import os, tarfile

def make_targz(output_filename, source_dir):
    with tarfile.open(output_filename, "w:gz") as tar:
        tar.add(source_dir, arcname=os.path.basename(source_dir))


make_targz('gamedata/gamedata', 'gamedata/json/')

